@@ -33,6 +33,11 @@
 #include "eom-close-confirmation-dialog.h"
 #include <eom-window.h>
 
+#if GTK_CHECK_VERSION (3, 2, 0)
+#define gtk_hbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,Y)
+#define gtk_vbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_VERTICAL,Y)
+#endif
+
 /* Properties */
 enum 
 {
@@ -394,13 +399,21 @@ build_single_img_dialog (EomCloseConfirmationDialog *dlg)
 	/* Image */
 	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, 
 					  GTK_ICON_SIZE_DIALOG);
+#if GTK_CHECK_VERSION (3, 14, 0)
+	gtk_widget_set_valign (image, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+#endif
 
 	/* Primary label */
 	primary_label = gtk_label_new (NULL);
 	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
 	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
+#if GTK_CHECK_VERSION (3, 14, 0)
+	gtk_widget_set_halign (primary_label, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (primary_label), 0.0, 0.5);
+#endif
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
 	image_name = eom_image_get_caption (img);
@@ -419,7 +432,11 @@ build_single_img_dialog (EomCloseConfirmationDialog *dlg)
 	secondary_label = gtk_label_new (str);
 	g_free (str);
 	gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
+#if GTK_CHECK_VERSION (3, 14, 0)
+	gtk_widget_set_halign (secondary_label, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (secondary_label), 0.0, 0.5);
+#endif
 	gtk_label_set_selectable (GTK_LABEL (secondary_label), TRUE);
 
 	hbox = gtk_hbox_new (FALSE, 12);
@@ -586,7 +603,11 @@ build_multiple_imgs_dialog (EomCloseConfirmationDialog *dlg)
 	/* Image */
 	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, 
 					  GTK_ICON_SIZE_DIALOG);
+#if GTK_CHECK_VERSION (3, 14, 0)
+	gtk_widget_set_valign (image, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+#endif
 	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
 
 	vbox = gtk_vbox_new (FALSE, 12);
@@ -596,7 +617,11 @@ build_multiple_imgs_dialog (EomCloseConfirmationDialog *dlg)
 	primary_label = gtk_label_new (NULL);
 	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
 	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
+#if GTK_CHECK_VERSION (3, 14, 0)
+	gtk_widget_set_halign (primary_label, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (primary_label), 0.0, 0.5);
+#endif
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
 	str = g_strdup_printf (
@@ -621,7 +646,11 @@ build_multiple_imgs_dialog (EomCloseConfirmationDialog *dlg)
 
 	gtk_box_pack_start (GTK_BOX (vbox2), select_label, FALSE, FALSE, 0);
 	gtk_label_set_line_wrap (GTK_LABEL (select_label), TRUE);
+#if GTK_CHECK_VERSION (3, 14, 0)
+	gtk_widget_set_halign (select_label, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (select_label), 0.0, 0.5);
+#endif
 
 	scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
 	gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow, TRUE, TRUE, 0);
@@ -640,7 +669,11 @@ build_multiple_imgs_dialog (EomCloseConfirmationDialog *dlg)
 
 	gtk_box_pack_start (GTK_BOX (vbox2), secondary_label, FALSE, FALSE, 0);
 	gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
+#if GTK_CHECK_VERSION (3, 14, 0)
+	gtk_widget_set_halign (secondary_label, GTK_ALIGN_START);
+#else
 	gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
+#endif
 	gtk_label_set_selectable (GTK_LABEL (secondary_label), TRUE);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (select_label), treeview);
