@@ -83,7 +83,8 @@ enum {
 static guint preview_signals [SIGNAL_LAST] = { 0 };
 
 enum {
-	PROP_IMAGE = 1,
+	PROP_0,
+	PROP_IMAGE,
 	PROP_IMAGE_X_ALIGN,
 	PROP_IMAGE_Y_ALIGN,
 	PROP_IMAGE_SCALE,
@@ -558,26 +559,32 @@ eom_print_preview_new (void)
 #if GTK_CHECK_VERSION (3, 0, 0)
 static gboolean
 draw_cb (GtkDrawingArea *drawing_area,
-	 cairo_t *cr,
+         cairo_t *cr,
+         gpointer  user_data)
+{
+	update_relative_sizes (EOM_PRINT_PREVIEW (user_data));
+
+	eom_print_preview_draw (EOM_PRINT_PREVIEW (user_data), cr);
+
+	if (cairo_status (cr) != CAIRO_STATUS_SUCCESS) {
+	    fprintf (stderr, "Cairo is unhappy: %s\n",
+	    cairo_status_to_string (cairo_status (cr)));
+	}
+	return TRUE;
+}
 #else
 static void
 expose_event_cb (GtkDrawingArea *drawing_area,
 		 GdkEventExpose *eev,
-#endif
 		 gpointer  user_data)
 {
 	GtkWidget *widget;
-#if !GTK_CHECK_VERSION (3, 0, 0)
 	cairo_t *cr;
-#endif
 
 	widget = GTK_WIDGET (drawing_area);
 
 	update_relative_sizes (EOM_PRINT_PREVIEW (user_data));
-
-#if !GTK_CHECK_VERSION (3, 0, 0)
 	cr = gdk_cairo_create (gtk_widget_get_window (widget));
-#endif
 
 	eom_print_preview_draw (EOM_PRINT_PREVIEW (user_data), cr);
 
@@ -585,16 +592,10 @@ expose_event_cb (GtkDrawingArea *drawing_area,
 		fprintf (stderr, "Cairo is unhappy: %s\n",
 			 cairo_status_to_string (cairo_status (cr)));
 	}
-
-#if !GTK_CHECK_VERSION (3, 0, 0)
 	cairo_destroy (cr);
-#endif
-
 	gdk_window_get_pointer (gtk_widget_get_window (widget), NULL, NULL, NULL);
-#if GTK_CHECK_VERSION (3, 0, 0)
-	return TRUE;
-#endif
 }
+#endif
 
 /**
  * get_current_image_coordinates:
@@ -977,7 +978,11 @@ motion_notify_event_cb (GtkWidget      *widget,
 							     GDK_FLEUR);
 			gdk_window_set_cursor (gtk_widget_get_window (widget),
 					       cursor);
+#if GTK_CHECK_VERSION (3, 0, 0)
+			g_object_unref (cursor);
+#else
 			gdk_cursor_unref (cursor);
+#endif
 		} else {
 			gdk_window_set_cursor (gtk_widget_get_window (widget),
 					       NULL);
@@ -1013,7 +1018,11 @@ eom_print_preview_draw (EomPrintPreview *preview, cairo_t *cr)
 	GtkWidget *area;
 	GtkAllocation allocation;
 	gint x0, y0;
+#if GTK_CHECK_VERSION (3, 4, 0)
+	GdkRGBA color;
+#else
 	GtkStyle *style;
+#endif
 	gboolean has_focus;
 
 	priv = preview->priv;
@@ -1021,17 +1030,29 @@ eom_print_preview_draw (EomPrintPreview *preview, cairo_t *cr)
 
 	has_focus = gtk_widget_has_focus (area);
 
+#if !GTK_CHECK_VERSION (3, 4, 0)
 	style = gtk_widget_get_style (area);
+#endif
 
 	gtk_widget_get_allocation (area, &allocation);
 
 	/* draw the page */
+#if GTK_CHECK_VERSION (3, 4, 0)
+	gdk_rgba_parse (&color, "white");
+	gdk_cairo_set_source_rgba (cr, &color);
+#else
  	gdk_cairo_set_source_color (cr, &style->white);
+#endif
  	cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
  	cairo_fill (cr);
 
 	/* draw the page margins */
+#if GTK_CHECK_VERSION (3, 4, 0)
+	gdk_rgba_parse (&color, "black");
+	gdk_cairo_set_source_rgba (cr, &color);
+#else
 	gdk_cairo_set_source_color (cr, &style->black);
+#endif
 	cairo_set_line_width (cr, 0.1);
 	cairo_rectangle (cr,
 			 priv->l_rmargin, priv->t_rmargin,
@@ -1073,15 +1094,17 @@ eom_print_preview_draw (EomPrintPreview *preview, cairo_t *cr)
 	}
 
 	if (has_focus) {
-		#if GTK_CHECK_VERSION(3, 0, 0)
-		gtk_paint_focus (style, cr,
-				 GTK_STATE_NORMAL, NULL, NULL,
-				 0, 0, allocation.width, allocation.height);
-		#else
+#if GTK_CHECK_VERSION(3, 0, 0)
+		GtkStyleContext *ctx;
+
+		ctx = gtk_widget_get_style_context (area);
+		gtk_render_focus (ctx, cr, 0, 0,
+				  allocation.width, allocation.height);
+#else
 		gtk_paint_focus (style, gtk_widget_get_window (area),
 				 GTK_STATE_NORMAL, NULL, NULL, NULL,
 				 0, 0, allocation.width, allocation.height);
-		#endif
+#endif
 	}
 }
 
